/*******************************************************************************
Copyright 2016 Microchip Technology Inc. (www.microchip.com)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

To request to license the code under the MLA license (www.microchip.com/mla_license), 
please contact mla_licensing@microchip.com
*******************************************************************************/

/** INCLUDES *******************************************************/
#include "system.h"

#include <stdint.h>
#include <string.h>
#include <stddef.h>

#include "usb.h"

#include "app_led_usb_status.h"
#include "app_device_cdc_basic.h"
#include "usb_config.h"

/** VARIABLES ******************************************************/

static bool buttonPressed;
static char buttonMessage[] = "Button pressed.\r\n";
static uint8_t readBuffer[CDC_DATA_OUT_EP_SIZE];
static uint8_t writeBuffer[CDC_DATA_IN_EP_SIZE];

unsigned int eeprom_addr=0;
unsigned char in_data=0;
unsigned char out_data=0;
uint8_t indata[10];
uint8_t outdata[10];
unsigned char ok_flg=0;
unsigned char out_byte=0;

/********************************************************************/
/*-------------------------WAIT----------------------------*/
void    wait_time(unsigned int time)
{
    while(time!=0){
        time--;
    }
}
/*------------------------EEPROM---------------------------*/
void    out_14bit(unsigned int data)
{
    unsigned char i;
    
    data=data << 2;
    for(i=0;i<14;i++){
        if((data&0x8000)==0x8000){
            EEPROM_DOUT=1;
        }
        else{
            EEPROM_DOUT=0;
        }
        wait_time(WAIT_E);
        EEPROM_CLK=1;
        wait_time(WAIT_E);
        EEPROM_CLK=0;
        data =data << 1;
        wait_time(WAIT_E);
    }
}
void    out_8bit(unsigned char data)
{
    unsigned char i;
    
    for(i=0;i<8;i++){
        if((data&0x80)==0x80){
            EEPROM_DOUT=1;
        }
        else{
            EEPROM_DOUT=0;
        }
        wait_time(WAIT_E);
        EEPROM_CLK=1;
        wait_time(WAIT_E);
        EEPROM_CLK=0;
        data =data << 1;
        wait_time(WAIT_E);
    }
}
unsigned char in_8bit(void)
{
    unsigned char i,data;
    data=0;
    for(i=0;i<8;i++){
        data=data<<1;        
        EEPROM_CLK=1;
        wait_time(WAIT_E);
        if(EEPROM_DIN==1){
            data=data|0x001;
        }
        EEPROM_CLK=0;
        wait_time(WAIT_E);
    }
    
    return data;
}
unsigned char eepromread(unsigned int addr)
{
    unsigned int adata;
    unsigned char rdata;
    
    EEPROM_DOUT=0;
    adata=addr;
    adata=adata | 0x3000;
    EEPROM_CS=1;
    out_14bit(adata);
    EEPROM_DOUT=0;
    wait_time(WAIT_E);
    rdata=in_8bit();
    EEPROM_CS=0;
    
    return rdata;
}
void    eeprom_ewen(void)
{
    unsigned int data;
    
    EEPROM_DOUT=1;
    data=0x2600;
    EEPROM_CS=1;
    out_14bit(data);
    EEPROM_DOUT=0;
    EEPROM_CS=0;
}
void    eeprom_ewds(void)
{
    unsigned int data;
    
    EEPROM_DOUT=1;
    data=0x2000;
    EEPROM_CS=1;
    out_14bit(data);
    EEPROM_DOUT=0;
    EEPROM_CS=0;
}
void    eepromwrite(unsigned int addr,unsigned char data)
{
    unsigned int adata;
    unsigned char wdata;
    
    EEPROM_DOUT=0;
    adata=addr;
    adata=adata | 0x2800;
    EEPROM_CS=1;
    out_14bit(adata);
    wdata=data;
    out_8bit(wdata);
    EEPROM_DOUT=0;
    EEPROM_CS=0;
    wait_time(WAIT_L);
    EEPROM_CS=1;
    while(EEPROM_DIN==0){}
    wait_time(WAIT_E);
    EEPROM_CS=0;
}
void    eeprom_erase(unsigned int addr)
{
    unsigned int adata;
    
    EEPROM_DOUT=0;
    adata=addr;
    adata=adata | 0x3800;
    EEPROM_CS=1;
    out_14bit(adata);
    EEPROM_DOUT=0;
    EEPROM_CS=0;
    wait_time(WAIT_L);
    EEPROM_CS=1;
    while(EEPROM_DIN==0){}
    wait_time(WAIT_E);
    EEPROM_CS=0;
}
void eeprom_eral(void)
{
    unsigned int data;
    
    EEPROM_DOUT=0;
    data=0x2400;
    EEPROM_CS=1;
    out_14bit(data);
    EEPROM_DOUT=0;
    EEPROM_CS=0;
    wait_time(WAIT_L);
    EEPROM_CS=1;
    while(EEPROM_DIN==0){}
    wait_time(WAIT_E);
    EEPROM_CS=0;
}
/*-----------------------8bit data => 3moji ASCII---------------*/
void    moji_henkan(unsigned char data)
{
    unsigned n0,n1,n2;

    n0=data%10;data=(data-n0)/10;
    n1=data%10;data=(data-n1)/10;
    n2=data%10;
    
    outdata[0]=0x53;
    outdata[1]=n2+0x30;
    outdata[2]=n1+0x30;
    outdata[3]=n0+0x30;
}
/*-----------------------16bit data => 4moji ASCII---------------*/
void    moji_henkan4(unsigned int data)
{
    unsigned n0,n1,n2,n3;

    n0=data%10;data=(data-n0)/10;
    n1=data%10;data=(data-n1)/10;
    n2=data%10;data=(data-n2)/10;
    n3=data%10;
    
    outdata[0]=0x51;
    outdata[1]=n3+0x30;
    outdata[2]=n2+0x30;
    outdata[3]=n1+0x30;
    outdata[4]=n0+0x30;
}
/*-------------------READ HENKAN---------------------------*/
void    henkan_read_data(void)
{
    unsigned char data;
    
    switch(readBuffer[0]){
        case 0x41:              //Axxxx
            LED1=LED_ON;
            eeprom_addr=(unsigned int)(readBuffer[1]-0x30)*1000+(unsigned int)(readBuffer[2]-0x30)*100+(unsigned int)(readBuffer[3]-0x30)*10+(unsigned int)(readBuffer[4]-0x30);
            LED1=LED_OFF;
            break;
        case 0x50:              //P
            LED1=LED_ON;
            moji_henkan4(eeprom_addr);
            out_byte=5;
            ok_flg=1;
            LED1=LED_OFF;
            break;
        case 0x57:              //Wxx
            LED1=LED_ON;
            in_data=(unsigned char)(readBuffer[1]-0x30)*100+(unsigned char)(readBuffer[2]-0x30)*10+(unsigned char)(readBuffer[3]-0x30);
            eepromwrite(eeprom_addr,in_data);
            outdata[0]=0x59;
            out_byte=1;
            ok_flg=1;
            //eeprom_addr++;
            LED1=LED_OFF;
            break;
        case 0x52:              //R
            LED1=LED_ON;
            data=eepromread(eeprom_addr);
            moji_henkan(data);
            out_byte=4;
            ok_flg=1;
            LED1=LED_OFF;
            break;
        case 0x45:              //E
            LED1=LED_ON;
            eeprom_erase(eeprom_addr);
            outdata[0]=0x59;
            out_byte=1;
            ok_flg=1;
            LED1=LED_OFF;
            break;
        case 0x46:              //F
            LED1=LED_ON;
            eeprom_eral();
            outdata[0]=0x59;
            out_byte=1;
            ok_flg=1;
            LED1=LED_OFF;
            break;
        case 0x47:              //G
            LED1=LED_ON;
            eeprom_ewen();
            LED1=LED_OFF;
            break;
        case 0x48:              //H
            LED1=LED_ON;
            eeprom_ewds();
            LED1=LED_OFF;
            break;
        case 0x49:              //I
            LED1=LED_ON;
            eeprom_addr++;
            if(eeprom_addr>2047){
                eeprom_addr=0;
            }
            LED1=LED_OFF;
            break;
    }
}

/*********************************************************************
* Function: void APP_DeviceCDCBasicDemoInitialize(void);
*
* Overview: Initializes the demo code
*
* PreCondition: None
*
* Input: None
*
* Output: None
*
********************************************************************/
void APP_DeviceCDCBasicDemoInitialize()
{   
    line_coding.bCharFormat = 0;
    line_coding.bDataBits = 8;
    line_coding.bParityType = 0;
    line_coding.dwDTERate = 115200;         //9600;

    buttonPressed = false;
}

/*********************************************************************
* Function: void APP_DeviceCDCBasicDemoTasks(void);
*
* Overview: Keeps the demo running.
*
* PreCondition: The demo should have been initialized and started via
*   the APP_DeviceCDCBasicDemoInitialize() and APP_DeviceCDCBasicDemoStart() demos
*   respectively.
*
* Input: None
*
* Output: None
*
********************************************************************/
void APP_DeviceCDCBasicDemoTasks()
{
    /* If the USB device isn't configured yet, we can't really do anything
     * else since we don't have a host to talk to.  So jump back to the
     * top of the while loop. */
    if( USBGetDeviceState() < CONFIGURED_STATE )
    {
        return;
    }

    /* If we are currently suspended, then we need to see if we need to
     * issue a remote wakeup.  In either case, we shouldn't process any
     * keyboard commands since we aren't currently communicating to the host
     * thus just continue back to the start of the while loop. */
    if( USBIsDeviceSuspended()== true )
    {
        return;
    }
        
    /* If the user has pressed the button associated with this demo, then we
     * are going to send a "Button Pressed" message to the terminal.
     */
    if(BUTTON_IsPressed(BUTTON_DEVICE_CDC_BASIC_DEMO) == true)
    {
        /* Make sure that we only send the message once per button press and
         * not continuously as the button is held.
         */
        if(buttonPressed == false)
        {
            /* Make sure that the CDC driver is ready for a transmission.
             */
            //if(mUSBUSARTIsTxTrfReady() == true)
            //{
            //    putrsUSBUSART(buttonMessage);
            //    buttonPressed = true;
            //}
            
            buttonPressed = true;
        }
    }
    else
    {
        /* If the button is released, we can then allow a new message to be
         * sent the next time the button is pressed.
         */
        buttonPressed = false;
    }

    /* Check to see if there is a transmission in progress, if there isn't, then
     * we can see about performing an echo response to data received.
     */
    if( USBUSARTIsTxTrfReady() == true)
    {
        uint8_t i;
        uint8_t numBytesRead;

        numBytesRead = getsUSBUSART(readBuffer, sizeof(readBuffer));

        /* For every byte that was read... */

        if(numBytesRead > 0)
        {
            /* After processing all of the received data, we need to send out
             * the "echo" data now.
             */
            //putUSBUSART(writeBuffer,numBytesRead);
            ok_flg=0;
            henkan_read_data();
        }
        if(ok_flg==1){
            putUSBUSART(outdata,out_byte);
            ok_flg=0;
        }
    }

    CDCTxService();
}